﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Security.Principal;
using System.IO;
using System.Security.AccessControl;
using System.ComponentModel;
using System.Xml;
using System.Reflection;
using ShellLinks;
using System.Text;
using System.Drawing;


namespace Send2FTP
{
    public partial class FormSend2FTPSettings : Form
    {
        public FormSend2FTPSettings()
        {
            InitializeComponent();
        }

        public static Icon GetExecutableIcon()
        {
            IntPtr large;
            IntPtr small;
            ExtractIconEx(Application.ExecutablePath, 0, out large, out small, 1);
            return Icon.FromHandle(small);
        }

        [DllImport("Shell32")]
        public static extern int ExtractIconEx(
            string sFile,
            int iIndex,
            out IntPtr piLargeVersion,
            out IntPtr piSmallVersion,
            int amountIcons);

        private String RemoteDirectoryHistory = "";

        private String WINDOWS_APPDATA = System.Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);

        private bool isReadingSettings = false;

        #region Settings - XML

        public static int key = 129;

        public static string EncryptDecrypt(string textToEncrypt)
        {
            StringBuilder inSb = new StringBuilder(textToEncrypt);
            StringBuilder outSb = new StringBuilder(textToEncrypt.Length);
            char c;
            for (int i = 0; i < textToEncrypt.Length; i++)
            {
                c = inSb[i];
                c = (char)(c ^ key);
                outSb.Append(c);
            }
            return outSb.ToString();
        }   

        #region hasWriteAccess
        [StructLayout(LayoutKind.Sequential)]
        internal struct GENERIC_MAPPING
        {
            internal uint GenericRead;
            internal uint GenericWrite;
            internal uint GenericExecute;
            internal uint GenericAll;
        }
        [DllImport("advapi32.dll", SetLastError = false)]
        static extern void MapGenericMask([In, MarshalAs(UnmanagedType.U4)] ref TokenAccessLevels AccessMask,
            [In] ref GENERIC_MAPPING map);

        [DllImport("advapi32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool DuplicateToken(IntPtr ExistingTokenHandle,
                [MarshalAs(UnmanagedType.U4)] TokenImpersonationLevel level,
                out int DuplicateTokenHandle);
        [DllImport("advapi32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool AccessCheck(
            [MarshalAs(UnmanagedType.LPArray)]
        byte[] pSecurityDescriptor,
            IntPtr ClientToken,
            [MarshalAs(UnmanagedType.U4)]
        TokenAccessLevels accessmask,
            [In] ref GENERIC_MAPPING GenericMapping,
            IntPtr PrivilegeSet,
            ref int PrivilegeSetLength,
            out uint GrantedAccess,
            [MarshalAs(UnmanagedType.Bool)]
      out bool AccessStatus);
        [DllImport("kernel32")]
        static extern void CloseHandle(IntPtr ptr);

        internal static bool hasWriteAccess(string path)
        {
            // Obtain the authenticated user's Identity

            WindowsIdentity winId = WindowsIdentity.GetCurrent(TokenAccessLevels.Duplicate | TokenAccessLevels.Query);

            WindowsImpersonationContext ctx = null;
            int statError = 0;
            IntPtr dupToken = IntPtr.Zero;
            try
            {
                // Start impersonating
                //ctx = winId.Impersonate(); works but AccessCheck does not like this

                int outPtr;
                //AccessCheck needs a duplicated token!
                DuplicateToken(winId.Token, TokenImpersonationLevel.Impersonation, out outPtr);

                dupToken = new IntPtr(outPtr);
                ctx = WindowsIdentity.Impersonate(dupToken);
                GENERIC_MAPPING map = new GENERIC_MAPPING();
                map.GenericRead = 0x80000000;
                map.GenericWrite = 0x40000000;
                map.GenericExecute = 0x20000000;
                map.GenericAll = 0x10000000;
                TokenAccessLevels required = TokenAccessLevels.Query | TokenAccessLevels.Read | TokenAccessLevels.Write | TokenAccessLevels.AssignPrimary | (TokenAccessLevels)0x00100000; // add synchronization
                MapGenericMask(ref required, ref map);


                uint status = 0;
                bool accesStatus = false;
                // dummy area the size should be 20 we don't do anything with it
                int sizeps = 20;
                IntPtr ps = Marshal.AllocCoTaskMem(sizeps);

                //AccessControlSections.Owner | AccessControlSections.Group MUST be included,
                //otherwise the descriptor would be seen with ERROR 1338
                var ACE = Directory.GetAccessControl(path,
                    AccessControlSections.Access | AccessControlSections.Owner |
                        AccessControlSections.Group);

                bool success = AccessCheck(ACE.GetSecurityDescriptorBinaryForm(), dupToken, required, ref map,
                        ps, ref sizeps, out status, out accesStatus);
                Marshal.FreeCoTaskMem(ps);
                if (!success)
                {
                    statError = Marshal.GetLastWin32Error();
                }
                else
                {
                    return accesStatus;
                }
            }
            // Prevent exceptions from propagating
            catch (Exception)
            {
                // Trace.Write(ex.Message);
            }
            finally
            {
                // Revert impersonation

                if (ctx != null)
                    ctx.Undo();
                CloseHandle(dupToken);
            }
            if (statError != 0)
            {
                throw new Win32Exception(statError);
            }

            return false;
        }
        #endregion

        private void _WriteSettingsXml(String path)
        {
            if (!hasWriteAccess(Path.GetDirectoryName(path)))
                throw new FileNotFoundException("XML Could not be written", path);

            XmlTextWriter textWriter = new XmlTextWriter(path, null);
            textWriter.Formatting = Formatting.Indented;

            textWriter.WriteStartDocument();

            textWriter.WriteComment("WhiteByte Send2FTP - Config File");

            textWriter.WriteStartElement("Settings");

            textWriter.WriteElementString("AskForDirectory", checkBoxAskDir.Checked.ToString());
            textWriter.WriteElementString("CopyURLToClipboard", checkBoxCpyUrl.Checked.ToString());
            textWriter.WriteElementString("ShowWebHost", checkBoxOpnWebHost.Checked.ToString());

            String CompressionMode = "0";
            if (radioButtonAskCmp.Checked)
                CompressionMode = "1";
            else if (radioButtonZIPCmp.Checked)
                CompressionMode = "2";

            textWriter.WriteElementString("CompressionMode", CompressionMode);

            textWriter.WriteStartElement("Servers");

            foreach (ListViewItem item in listView1.Items)
            {
                String[] obj = (String[])item.Tag;

                textWriter.WriteStartElement(obj[2] + "-" + obj[0]);

                textWriter.WriteElementString("Username", obj[0]);
                textWriter.WriteElementString("Password", EncryptDecrypt(obj[1]));
                textWriter.WriteElementString("Host", obj[2]);

                textWriter.WriteElementString("ShowEntry", obj[3]);
                textWriter.WriteElementString("RemoteDirectory", obj[4]);
                textWriter.WriteElementString("WebHost", obj[5]);

                textWriter.WriteEndElement();
            }


            textWriter.WriteEndElement();

            textWriter.WriteElementString("RemoteDirectoryHistory", RemoteDirectoryHistory);

            textWriter.WriteEndElement();
            textWriter.WriteEndDocument();
            textWriter.Close();

            if (!File.Exists(path))
                throw new FileNotFoundException("Could not Write XML", path);

        }

        private void WriteSettingsXml2()
        {
            try
            {
                Directory.CreateDirectory(WINDOWS_APPDATA + @"\Send2FTP\");
                _WriteSettingsXml(WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml");
            }
            catch
            {
                MessageBox.Show("Sorry could not create the Settings file. Will Close the Application now.");
                Environment.Exit(0);
            }
        }

        private bool WriteSettingsXml()
        {
            if (isReadingSettings)
                return false;

            if (File.Exists(WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml"))
            {
                WriteSettingsXml2();
            }
            else
            {
                try
                {
                    _WriteSettingsXml(Environment.CurrentDirectory + @"\Send2FTPConfig.xml");
                }
                catch
                {
                    DialogResult result = MessageBox.Show(@"The Settings File could not be created in " + Environment.CurrentDirectory + @"\Send2FTPConfig.xml" + ", do you want me to try to create it in " + WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml ?", "Send2FTP", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        WriteSettingsXml2();
                    }
                    else
                    {
                        Environment.Exit(0);
                    }
                }
            }

            return true;
        }

        private void ReadSettingsXml()
        {
            isReadingSettings = true;

            String path = null;

            if (File.Exists(WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml"))
                path = WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml";
            else if (hasWriteAccess(Environment.CurrentDirectory) && File.Exists("Send2FTPConfig.xml"))
                path = Environment.CurrentDirectory + @"\Send2FTPConfig.xml";

            if (path != null)
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(path);

                XmlElement root = doc.DocumentElement;

                XmlNode node;

                node = root.SelectSingleNode("/Settings/AskForDirectory");
                checkBoxAskDir.Checked = node.InnerText.Equals("True");

                node = root.SelectSingleNode("/Settings/CopyURLToClipboard");
                checkBoxCpyUrl.Checked = node.InnerText.Equals("True");

                node = root.SelectSingleNode("/Settings/ShowWebHost");
                checkBoxOpnWebHost.Checked = node.InnerText.Equals("True");

                node = root.SelectSingleNode("/Settings/RemoteDirectoryHistory");

                if (node != null)
                    RemoteDirectoryHistory = node.InnerText;

                node = root.SelectSingleNode("/Settings/CompressionMode");

                if (node.InnerText.Equals("1"))
                    radioButtonAskCmp.Checked = true;
                else if (node.InnerText.Equals("2"))
                    radioButtonZIPCmp.Checked = true;

                foreach (XmlElement Server in doc.SelectNodes("Settings/Servers/*"))
                {
                    ListViewItem lvi = new ListViewItem();

                    lvi.Text = Server["Host"].InnerText;
                    lvi.SubItems.Add(Server["Username"].InnerText);

                    lvi.Tag = new String[] { Server["Username"].InnerText, EncryptDecrypt(Server["Password"].InnerText), Server["Host"].InnerText, 
                                             Server["ShowEntry"].InnerText, Server["RemoteDirectory"].InnerText, Server["WebHost"].InnerText };

                    listView1.Items.Add(lvi);
                }
            }

            isReadingSettings = false;
        }
        #endregion

        private void Form1_Load(object sender, EventArgs e)
        {
            lblVersion.Text = Assembly.GetExecutingAssembly().GetName().Version.ToString();
            this.Icon = GetExecutableIcon();

            ReadSettingsXml();
        }

        private void Settings_CheckedChanged(object sender, EventArgs e)
        {
            WriteSettingsXml();
        }

        public bool LviExists(String Host, String Usr)
        {
            foreach (ListViewItem lvi in listView1.Items)
            {
                if ((lvi.Text == Host) && (lvi.SubItems[1].Text == Usr))
                    return true;
            }

            return false;
        }

        private void buttonAddFTP_Click(object sender, EventArgs e)
        {
            if (LviExists(textBoxHost.Text, textBoxUsr.Text))
            {
                MessageBox.Show("Sorry you can't add two Servers with the Same Username and the same Host");
                return;
            }

            ListViewItem tmp = new ListViewItem();

            tmp.Text = textBoxHost.Text;
            tmp.SubItems.Add(textBoxUsr.Text);

            tmp.Tag = new String[] { textBoxUsr.Text, textBoxPwd.Text, textBoxHost.Text, checkBoxShowInSndTo.Checked.ToString(), textBoxRemoteDir.Text, textBoxWebHost.Text };

            listView1.Items.Add(tmp);

            WriteSettingsXml();

            if (checkBoxShowInSndTo.Checked)
            {
                String LinkName = "Send to " + textBoxHost.Text + " as " + textBoxUsr.Text;
                String param = textBoxHost.Text + "-" + textBoxUsr.Text;

                if (!Link.Exists(Environment.SpecialFolder.SendTo, LinkName))
                    Link.Update(Environment.SpecialFolder.SendTo, Application.ExecutablePath, LinkName, true, param);
            }
        }

        private void listView1_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (e != null)
            {
                String[] str = (String[])e.Item.Tag;

                textBoxUsr.Text = str[0];
                textBoxPwd.Text = str[1];
                textBoxHost.Text = str[2];
                checkBoxShowInSndTo.Checked = (str[3] == "True");
                textBoxRemoteDir.Text = str[4];
                textBoxWebHost.Text = str[5];
            }
        }

        private void buttonUpdtFTP_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                ListViewItem tmp = listView1.SelectedItems[0];

                tmp.SubItems.Clear();

                tmp.Text = textBoxHost.Text;
                tmp.SubItems.Add(textBoxUsr.Text);

                tmp.Tag = new String[] { textBoxUsr.Text, textBoxPwd.Text, textBoxHost.Text, checkBoxShowInSndTo.Checked.ToString(), textBoxRemoteDir.Text, textBoxWebHost.Text };

                WriteSettingsXml();
            }
        }

        private void buttonRemoveFTP_Click(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                if (checkBoxShowInSndTo.Checked)
                    checkBoxShowInSndTo.Checked = false;

                listView1.Items.Remove(listView1.SelectedItems[0]);
                WriteSettingsXml();
            }
        }

        private void checkBoxShowInSndTo_CheckedChanged(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                String LinkName = "Send to " + textBoxHost.Text + " as " + textBoxUsr.Text;
                String param = textBoxHost.Text + "-" + textBoxUsr.Text;

                if (checkBoxShowInSndTo.Checked)
                {
                    if (!Link.Exists(Environment.SpecialFolder.SendTo, LinkName))
                        Link.Update(Environment.SpecialFolder.SendTo, Application.ExecutablePath, LinkName, true, param);
                }
                else
                {
                    if (Link.Exists(Environment.SpecialFolder.SendTo, LinkName))
                        Link.Update(Environment.SpecialFolder.SendTo, Application.ExecutablePath, LinkName, false, param);
                }

                buttonUpdtFTP_Click(null, null);
            }
        }

        private void textBoxHost_TextChanged(object sender, EventArgs e)
        {
            if ((textBoxHost.Text.Length > 0) && (textBoxHost.Text.Substring(0, textBoxHost.Text.Length - 1) == textBoxWebHost.Text))
            {
                textBoxWebHost.Text = textBoxHost.Text;
            }
        }
    }
}