﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Threading;
using FTPLib;
using Ionic.Zip;

namespace Send2FTP
{
    public partial class FormSend2FTPUploader : Form
    {
        private String[] args;

        public FormSend2FTPUploader(String[] args)
        {
            InitializeComponent();

            this.args = args;
        }

        private String WINDOWS_APPDATA = System.Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);

        bool isAskDir = false;
        bool isCpyUrl = false;
        bool isOpnWebHost = false;

        String compressionMode = "0"; 

        String username;
        String host;
        String password;
        String RemoteDirectory;
        String WebHost;
        String RemoteDirectoryHistory = "";

        String UploadPath;

        Thread uploader;

        private void ReadSettingsXml()
        {
            String path = null;

            String currDir = new FileInfo(Application.ExecutablePath).DirectoryName;

            if (File.Exists(WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml"))
                path = WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml";
            else if (FormSend2FTPSettings.hasWriteAccess(currDir) && File.Exists(currDir + @"\Send2FTPConfig.xml"))
                path = currDir + @"\Send2FTPConfig.xml";

            if (path != null)
            {
                XmlDocument doc = new XmlDocument();
                doc.Load(path);

                XmlElement root = doc.DocumentElement;

                XmlNode node;

                node = root.SelectSingleNode("/Settings/AskForDirectory");
                isAskDir = node.InnerText.Equals("True");

                node = root.SelectSingleNode("/Settings/CopyURLToClipboard");
                isCpyUrl = node.InnerText.Equals("True");

                node = root.SelectSingleNode("/Settings/ShowWebHost");
                isOpnWebHost = node.InnerText.Equals("True");

                node = root.SelectSingleNode("/Settings/CompressionMode");
                compressionMode = node.InnerText;

                try
                {
                    node = root.SelectSingleNode("/Settings/RemoteDirectoryHistory");
                    RemoteDirectoryHistory = node.InnerText;
                }
                catch { }

                XmlElement Server = (XmlElement)doc.SelectSingleNode("Settings/Servers/" + args[0]);

                username = Server["Username"].InnerText;
                password = FormSend2FTPSettings.EncryptDecrypt(Server["Password"].InnerText);
                host = Server["Host"].InnerText;
                RemoteDirectory = Server["RemoteDirectory"].InnerText;

                WebHost = Server["WebHost"].InnerText;
            }
        }

        private void FormSend2FTPUploader_Load(object sender, EventArgs e)
        {
            this.Icon = FormSend2FTPSettings.GetExecutableIcon();
            
            buttonShowDetails_Click(null, null);
            ReadSettingsXml();

            comboBoxRemoteDir.Items.AddRange(RemoteDirectoryHistory.Split("~".ToCharArray(), StringSplitOptions.RemoveEmptyEntries));

            comboBoxRemoteDir.Text = RemoteDirectory;
            comboBoxRemoteDir.Enabled = isAskDir;

            if (compressionMode.Equals("0"))
            {
                checkBoxCompress.Enabled = false;
            }
            else if (compressionMode.Equals("2"))
            {
                checkBoxCompress.Checked = true;
                checkBoxCompress.Enabled = false;
            }

            if ((!compressionMode.Equals("1") && !isAskDir))
            {
                buttonUpload_Click(null, null);
            }
        }

        private void buttonShowDetails_Click(object sender, EventArgs e)
        {
            if (groupBoxDetails.Visible)
            {
                this.Height = this.Height - (groupBoxDetails.Height + groupBoxDetails.Top);
                panel1.Top = 5;
            }
            else
            {
                this.Height = this.Height + (groupBoxDetails.Height + groupBoxDetails.Top);
                panel1.Top = (groupBoxDetails.Height + groupBoxDetails.Top);
            }

            groupBoxDetails.Visible = !groupBoxDetails.Visible;
        }

        private void buttonStop_Click(object sender, EventArgs e)
        {
            if (checkBoxCompress.Checked)
            {
                File.Delete(UploadPath);
            }

            Application.Exit();
        }

        public bool IsFolder(string path)
        {
            return ((File.GetAttributes(path) & FileAttributes.Directory) == FileAttributes.Directory);
        }

        public static FTP ftplib = null;

        private long deltabytes = 0;
        private TimeSpan deltatime = new TimeSpan(0);

        private long updateTime = 0;

        public void ProgressEvent(System.IO.FileInfo File, long BytesTotal, long FileSize, long TotalBytesDirectorySent, long TotalBytesDirectory, long StartTickCount)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { ProgressEvent(File, BytesTotal, FileSize, TotalBytesDirectorySent, TotalBytesDirectory, StartTickCount); }));
                return;
            }

            TimeSpan elapsedSpan = new TimeSpan(DateTime.Now.Ticks - StartTickCount);

            labelFilename.Text = "Filename: " + File.FullName;
            labelCurrentSend.Text = "Current: " + (BytesTotal / 1024).ToString() + "/" + (FileSize / 1024).ToString() + " KB";
            labelTotal.Text = "Total: " + (TotalBytesDirectorySent / 1024).ToString() + "/" + (TotalBytesDirectory / 1024).ToString() + " KB";
            labelElapsed.Text = "Elapsed: " + string.Format("{0:D2}:{1:D2}:{2:D2}", elapsedSpan.Hours, elapsedSpan.Minutes, elapsedSpan.Seconds);

            if (progressBar1.Value != (int)((BytesTotal * 100) / FileSize))
                progressBar1.Value = (int)((BytesTotal * 100) / FileSize);

            if (progressBar2.Value != (int)((TotalBytesDirectorySent * 100) / TotalBytesDirectory))
                progressBar2.Value = (int)((TotalBytesDirectorySent * 100) / TotalBytesDirectory);

            if ((new TimeSpan(DateTime.Now.Ticks - updateTime).TotalMilliseconds > 200) || (TotalBytesDirectorySent == TotalBytesDirectory))
            {
                updateTime = DateTime.Now.Ticks;

                deltabytes = TotalBytesDirectorySent - deltabytes;
                deltatime = elapsedSpan.Subtract(deltatime);

                if (deltatime.Seconds > 0)
                {
                    labelSpeed.Text = "Speed: " + (deltabytes / deltatime.Seconds / 1024).ToString() + " KB/s";

                    TimeSpan t = TimeSpan.FromSeconds((TotalBytesDirectory - deltabytes) / (deltabytes / deltatime.Seconds));
                    labelRemaining.Text = "Remaining: " + string.Format("{0:D2}:{1:D2}:{2:D2}", t.Hours, t.Minutes, t.Seconds);
                }
            }

            if (BytesTotal == FileSize)
            {
                deltabytes = 0;
                deltatime = new TimeSpan(0);
            }
        }

        public void ZipSaveProgress(object sender, SaveProgressEventArgs e)
        {
            if (e.EventType == ZipProgressEventType.Saving_Started)
            {
                labelFilename.Text = "Archiv File Name: " + e.ArchiveName;
            }
            else if (e.EventType == ZipProgressEventType.Saving_BeforeWriteEntry)
            {
                if (progressBar2.Maximum != e.EntriesTotal)
                    progressBar2.Maximum = e.EntriesTotal;

                labelCurrentSend.Text = String.Format(" Writing: {0} ({1}/{2})", e.CurrentEntry.FileName, e.EntriesSaved + 1, e.EntriesTotal);

                progressBar2.Value = e.EntriesSaved + 1;
            }
            else if (e.EventType == ZipProgressEventType.Saving_EntryBytesRead)
            {
                progressBar1.Value = (int)((e.BytesTransferred * 100) / e.TotalBytesToTransfer);
            }
            else if (e.EventType == ZipProgressEventType.Saving_Completed)
            {
                progressBar2.Maximum = 100;
            }
        }
        
        private void buttonUpload_Click(object sender, EventArgs e)
        {
            if (!WebHost.ToLower().Contains("http://"))
                WebHost = "http://" + WebHost;

            if (comboBoxRemoteDir.Text.Length > 0)
            {
                if (comboBoxRemoteDir.Text[0] != '/')
                {
                    comboBoxRemoteDir.Text = "/" + comboBoxRemoteDir.Text;
                }

                if (comboBoxRemoteDir.Text[comboBoxRemoteDir.Text.Length - 1] != '/')
                {
                    comboBoxRemoteDir.Text += "/";
                }
            }

            buttonUpload.Enabled = false;

            if (!groupBoxDetails.Visible)
                buttonShowDetails_Click(null, null);

            if (!comboBoxRemoteDir.Items.Contains(comboBoxRemoteDir.Text.ToLower()))
            {
                comboBoxRemoteDir.Items.Add(comboBoxRemoteDir.Text.ToLower());

                String path = null;

                String currDir = new FileInfo(Application.ExecutablePath).DirectoryName;

                if (File.Exists(WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml"))
                    path = WINDOWS_APPDATA + @"\Send2FTP\Send2FTPConfig.xml";
                else if (FormSend2FTPSettings.hasWriteAccess(currDir) && File.Exists(currDir + @"\Send2FTPConfig.xml"))
                    path = currDir + @"\Send2FTPConfig.xml";

                if (path != null)
                {
                    String[] sa_tmp = new String[comboBoxRemoteDir.Items.Count];
                    comboBoxRemoteDir.Items.CopyTo(sa_tmp, 0);

                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.Load(path);

                    XmlNode NodeRemoteDirectoryHistory = xmlDoc.SelectSingleNode("Settings/RemoteDirectoryHistory");

                    if (NodeRemoteDirectoryHistory == null)
                    {
                        NodeRemoteDirectoryHistory = xmlDoc.CreateElement("RemoteDirectoryHistory");
                        xmlDoc.SelectSingleNode("Settings").AppendChild(NodeRemoteDirectoryHistory);
                    }

                    NodeRemoteDirectoryHistory.InnerText = String.Join("~", sa_tmp);                    

                    xmlDoc.Save(path);
                }
            }

            uploader = new Thread(delegate()
            {
                ftplib = new FTP(host, username, password);

                ftplib.Connect();

                ftplib.tryMakeDir(comboBoxRemoteDir.Text);

                ftplib.ChangeDir(comboBoxRemoteDir.Text);

                UploadPath = args[1];

                if (checkBoxCompress.Checked)
                {
                    using (ZipFile zip = new ZipFile())
                    {
                        zip.CompressionLevel = Ionic.Zlib.CompressionLevel.Default;
                        zip.SaveProgress += ZipSaveProgress;

                        zip.StatusMessageTextWriter = System.Console.Out;

                        if (IsFolder(UploadPath))
                        {
                            zip.AddDirectory(UploadPath); // recurses subdirectories
                        }
                        else
                        {
                            zip.AddFile(UploadPath);
                        }


                        zip.Save(UploadPath + ".zip");
                    }

                    UploadPath = UploadPath + ".zip";
                }

                if (IsFolder(UploadPath))
                {
                    ftplib.UploadFolderRecursively(UploadPath, ProgressEvent);
                }
                else
                {
                    ftplib.UploadFile(UploadPath, ProgressEvent);
                }

                if (isOpnWebHost)
                {
                    if (IsFolder(UploadPath))
                    {
                        System.Diagnostics.Process.Start(WebHost + comboBoxRemoteDir.Text + new DirectoryInfo(UploadPath).Name);
                    }
                    else
                    {
                        System.Diagnostics.Process.Start(WebHost + comboBoxRemoteDir.Text);
                    }
                }

                if (this.InvokeRequired)
                {
                    this.BeginInvoke(new MethodInvoker(delegate()
                    {
                        if (isCpyUrl)
                        {
                            if (IsFolder(UploadPath))
                            {
                                Clipboard.SetText(WebHost + comboBoxRemoteDir.Text + new DirectoryInfo(UploadPath).Name);
                            }
                            else
                            {
                                Clipboard.SetText(WebHost + comboBoxRemoteDir.Text + new FileInfo(UploadPath).Name);
                            }
                        }
                    }));
                }

                buttonStop_Click(null, null);

            });

            uploader.Start();
        }
    }
}